@@ -1,25 +1,19 @@
-
-use std::collections::HashMap;
-use rocket::serde::{ Serialize, Deserialize };
 use rocket::response::{Flash, Redirect};
+use rocket::serde::{Deserialize, Serialize};
+use std::collections::HashMap;
 
 use rocket::http::{Cookie, CookieJar};
 
+use rocket::form::{Context, Contextual, Form, FromForm};
 use rocket::State;
-use rocket::form::{
-    Form,
-    Contextual,
-    FromForm,
-    Context,
-};
 use rocket_dyn_templates::Template;
 
-use printnanny_services::printnanny_api::{ ApiConfig, ApiService, ServiceError };
 use printnanny_api_client::models;
+use printnanny_services::config::{ApiConfig, PrintNannyConfig};
+use printnanny_services::printnanny_api::{ApiService, ServiceError};
 
-use super::config::{ Config };
 use super::error;
-use super::response::{ FlashResponse, Response };
+use super::response::{FlashResponse, Response};
 
 pub const COOKIE_API_CONFIG: &str = "printnanny_api_config";
 // generic
@@ -31,12 +25,6 @@ pub struct DashContext {
     // system_info: models::SystemInfo
 }
 
-// impl fmt::Display for ContextKey {
-//     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-//         write!(f, "{:?}", self)
-//     }
-// }
-
 #[derive(Debug, FromForm)]
 pub struct EmailForm<'v> {
     #[field(validate = contains('@').or_else(msg!("invalid email address")))]
@@ -50,18 +38,20 @@ pub struct TokenForm<'v> {
     token: &'v str,
 }
 
-async fn handle_step1(form: &EmailForm<'_>, config: &Config) -> Result<Response, FlashResponse<Template>> {
-    let api_config = ApiConfig{base_path: config.base_url.to_string(), bearer_access_token: None};
-    let service = ApiService::new(api_config, &config.path)?;
+async fn handle_step1(
+    form: &EmailForm<'_>,
+    config: &State<PrintNannyConfig>,
+) -> Result<Response, FlashResponse<Template>> {
+    let c = config.inner().clone();
+    let service = ApiService::new(c)?;
     let res = service.auth_email_create(form.email.to_string()).await;
     match res {
         Ok(_) => {
             let redirect = Redirect::to(format!("/login/{}", form.email));
             Ok(Response::Redirect(redirect))
-
-        },
+        }
         Err(e) => {
-            error!("{}",e);
+            error!("{}", e);
             let mut context = HashMap::new();
             context.insert("errors", format!("Something went wrong {:?}", e));
             Ok(Response::Template(Template::render("error", context)))
@@ -72,17 +62,25 @@ async fn handle_step1(form: &EmailForm<'_>, config: &Config) -> Result<Response,
 // fields to re-render forms with submitted values on error. If you have no such
 // need, do not use `Contextual`. Use the equivalent of `Form<Submit<'_>>`.
 #[post("/", data = "<form>")]
-async fn login_step1_submit<'r>(form: Form<Contextual<'r, EmailForm<'r>>>, config: &State<Config>) ->  Result<Response, FlashResponse<Template>> {
+async fn login_step1_submit<'r>(
+    form: Form<Contextual<'r, EmailForm<'r>>>,
+    config: &State<PrintNannyConfig>,
+) -> Result<Response, FlashResponse<Template>> {
     info!("Received auth email form response {:?}", form);
+    let c = config.inner().clone();
+
     match &form.value {
         Some(signup) => {
             let result = handle_step1(signup, config).await?;
             Ok(result)
-        },
+        }
         None => {
             info!("form.value is empty");
-            Ok(Response::Template(Template::render("authemail", &form.context)))
-        },
+            Ok(Response::Template(Template::render(
+                "authemail",
+                &form.context,
+            )))
+        }
     }
 }
 
@@ -90,17 +88,27 @@ async fn login_step1_submit<'r>(form: Form<Contextual<'r, EmailForm<'r>>>, confi
 // fields to re-render forms with submitted values on error. If you have no such
 // need, do not use `Contextual`. Use the equivalent of `Form<Submit<'_>>`.
 
-async fn handle_token_validate(token: &str, email: &str, config_path: &str, base_url: &str) -> Result<ApiConfig, ServiceError>{
-    let api_config = ApiConfig{base_path: base_url.to_string(), bearer_access_token: None};
-    
-    let service = ApiService::new(api_config, config_path)?;
+async fn handle_token_validate(
+    token: &str,
+    email: &str,
+    config: PrintNannyConfig,
+) -> Result<ApiConfig, ServiceError> {
+    let mut service = ApiService::new(config)?;
+    service.config.api = ApiConfig {
+        base_path: service.config.api.base_path,
+        bearer_access_token: None,
+    };
     let res = service.auth_token_validate(email, token).await?;
     let bearer_access_token = res.token.to_string();
+    info!("Success! Authenticated and received bearer token");
 
-    let api_config = ApiConfig{base_path: base_url.to_string(), bearer_access_token: Some(bearer_access_token)};
-    let service = ApiService::new(api_config.clone(), config_path)?;
+    service.config.api = ApiConfig {
+        base_path: service.config.api.base_path,
+        bearer_access_token: Some(bearer_access_token),
+    };
+    info!("Setting up device");
     service.device_setup().await?;
-    Ok(api_config)
+    Ok(service.config.api)
 }
 
 #[post("/<email>", data = "<form>")]
@@ -108,23 +116,48 @@ async fn login_step2_submit<'r>(
     email: String,
     jar: &CookieJar<'_>,
     form: Form<Contextual<'r, TokenForm<'r>>>,
-    config: &State<Config>) -> Result<FlashResponse<Redirect>, FlashResponse<Template>> {
+    config: &State<PrintNannyConfig>,
+) -> Result<FlashResponse<Redirect>, FlashResponse<Template>> {
     info!("Received auth email form response {:?}", form);
+    let c = config.inner().clone();
     match form.value {
         Some(ref v) => {
             let token = v.token;
-            let api_config = handle_token_validate(token, &email, &config.path, &config.base_url).await?;
+            let api_config = handle_token_validate(token, &email, c).await?;
             let cookie_value = serde_json::to_string(&api_config)?;
             jar.add_private(Cookie::new(COOKIE_API_CONFIG, cookie_value));
-            Ok(FlashResponse::<Redirect>::from(Flash::success(Redirect::to("/"), "Verification Success")))
-        },
+            Ok(FlashResponse::<Redirect>::from(Flash::success(
+                Redirect::to("/login/welcome"),
+                "Verification Success",
+            )))
+        }
         None => {
             info!("form.value is empty");
-            Err(FlashResponse::<Template>::from(error::Error::VerificationFailed{}))
-        },
+            Err(FlashResponse::<Template>::from(
+                error::Error::VerificationFailed {},
+            ))
+        }
     }
 }
 
+#[get("/welcome")]
+async fn login_step3(
+    jar: &CookieJar<'_>,
+    config: &State<PrintNannyConfig>,
+) -> Result<Response, FlashResponse<Template>> {
+    let get_api_config = jar.get_private(COOKIE_API_CONFIG);
+    match get_api_config {
+        Some(cookie) => {
+            let api_config: ApiConfig = serde_json::from_str(cookie.value())?;
+            let context = get_context(config).await?;
+            Ok(Response::Template(Template::render("welcome", context)))
+        }
+        None => Ok(Response::Template(Template::render(
+            "authemail",
+            &Context::default(),
+        ))),
+    }
+}
 
 #[get("/<email>")]
 fn login_step2(email: String) -> Template {
@@ -133,13 +166,12 @@ fn login_step2(email: String) -> Template {
     Template::render("authtoken", context)
 }
 
-pub async fn get_context(config_path: &str, api_config: &ApiConfig) -> Result<DashContext, ServiceError> {
-    let mut service = ApiService::new(api_config.clone(), config_path)?;
-    // user into context
-    let context = DashContext{
-        user: service.user.unwrap(),
-        device: service.device.unwrap(),
-    };
+pub async fn get_context(config: &State<PrintNannyConfig>) -> Result<DashContext, ServiceError> {
+    let c = config.inner().clone();
+    let service = ApiService::new(c)?;
+    let device = service.device_retrieve_hostname().await?;
+    let user = service.auth_user_retreive().await?;
+    let context = DashContext { user, device };
 
     Ok(context)
 }
@@ -148,10 +180,11 @@ pub async fn get_context(config_path: &str, api_config: &ApiConfig) -> Result<Da
 async fn login_step1(jar: &CookieJar<'_>) -> Result<Response, FlashResponse<Redirect>> {
     let get_api_config = jar.get_private(COOKIE_API_CONFIG);
     match get_api_config {
-        Some(_) => {
-            Ok(Response::Redirect(Redirect::to("/")))
-        },
-        None => Ok(Response::Template(Template::render("authemail", &Context::default())))
+        Some(_) => Ok(Response::Redirect(Redirect::to("/"))),
+        None => Ok(Response::Template(Template::render(
+            "authemail",
+            &Context::default(),
+        ))),
     }
 }
 
@@ -161,5 +194,6 @@ pub fn routes() -> Vec<rocket::Route> {
         login_step1_submit,
         login_step2,
         login_step2_submit,
+        login_step3
     ]
-}
\ No newline at end of file
+}
