@@ -1,6 +1,6 @@
-#[macro_use] extern crate rocket;
+#[macro_use]
+extern crate rocket;
 pub mod auth;
-pub mod config;
 pub mod error;
 pub mod home;
 pub mod response;
